@@ -61,7 +61,7 @@ class Wrapper:
         '    sizeof(%(tp_basicsize)s),	        /* tp_basicsize */\n' \
         '    0,					/* tp_itemsize */\n' \
         '    /* methods */\n' \
-        '    (destructor)0,			/* tp_dealloc */\n' \
+        '    (destructor)%(tp_dealloc)s,	/* tp_dealloc */\n' \
         '    (printfunc)0,			/* tp_print */\n' \
         '    (getattrfunc)%(tp_getattr)s,	/* tp_getattr */\n' \
         '    (setattrfunc)%(tp_setattr)s,	/* tp_setattr */\n' \
@@ -76,10 +76,10 @@ class Wrapper:
         '    (getattrofunc)0,			/* tp_getattro */\n' \
         '    (setattrofunc)0,			/* tp_setattro */\n' \
         '    (PyBufferProcs*)%(tp_as_buffer)s,	/* tp_as_buffer */\n' \
-        '    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /* tp_flags */\n' \
+        '    %(tp_flags)s,                      /* tp_flags */\n' \
         '    NULL, 				/* Documentation string */\n' \
-        '    (traverseproc)0,			/* tp_traverse */\n' \
-        '    (inquiry)0,			/* tp_clear */\n' \
+        '    (traverseproc)%(tp_traverse)s,	/* tp_traverse */\n' \
+        '    (inquiry)%(tp_clear)s,		/* tp_clear */\n' \
         '    (richcmpfunc)%(tp_richcompare)s,	/* tp_richcompare */\n' \
         '    %(tp_weaklistoffset)s,             /* tp_weaklistoffset */\n' \
         '    (getiterfunc)%(tp_iter)s,		/* tp_iter */\n' \
@@ -103,7 +103,8 @@ class Wrapper:
                   'tp_as_number', 'tp_as_sequence', 'tp_as_mapping', 'tp_hash',
                   'tp_call', 'tp_str', 'tp_as_buffer', 'tp_richcompare', 'tp_iter',
                   'tp_iternext', 'tp_descr_get', 'tp_descr_set', 'tp_init',
-                  'tp_alloc', 'tp_new', 'tp_free', 'tp_is_gc']
+                  'tp_alloc', 'tp_new', 'tp_free', 'tp_is_gc',
+                  'tp_traverse', 'tp_clear', 'tp_dealloc', 'tp_flags']
 
     getter_tmpl = \
         'static PyObject *\n' \
@@ -174,6 +175,8 @@ class Wrapper:
     def write_class(self):
         self.fp.write('\n/* ----------- ' + self.objinfo.c_name + ' ----------- */\n\n')
         substdict = self.get_initial_class_substdict()
+        if not substdict.has_key('tp_flags'):
+            substdict['tp_flags'] = 'Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE'
         substdict['typename'] = self.objinfo.c_name
         if self.overrides.modulename:
             substdict['classname'] = '%s.%s' % (self.overrides.modulename,
@@ -187,11 +190,9 @@ class Wrapper:
             substdict['tp_init'] = self.write_constructor()
         substdict['tp_methods'] = self.write_methods()
         substdict['tp_getset'] = self.write_getsets()
-
+            
         # handle slots ...
         for slot in self.slots_list:
-            if substdict.has_key(slot) and substdict[slot] != '0':
-                continue
             
             slotname = '%s.%s' % (self.objinfo.c_name, slot)
             slotfunc = '_wrap_%s_%s' % (self.get_lower_name(), slot)
@@ -202,7 +203,8 @@ class Wrapper:
                 self.write_function(slotname, data)
                 substdict[slot] = slotfunc
             else:
-                substdict[slot] = '0'
+                if not substdict.has_key(slot):
+                    substdict[slot] = '0'
     
         self.fp.write(self.type_tmpl % substdict)
 
@@ -617,7 +619,12 @@ class GObjectWrapper(Wrapper):
               '    if (!self->obj) {\n' \
               '        PyErr_SetString(PyExc_RuntimeError, "could not create %(typename)s object");\n' \
               '        return -1;\n' \
-              '    }\n' \
+              '    }\n'
+
+        if not constructor.caller_owns_return:
+            print >> out, "    g_object_ref(self->obj);\n"
+
+        print >> out, \
               '    pygobject_register_wrapper((PyObject *)self);\n' \
               '    return 0;\n' \
               '}\n\n' % { 'typename': classname }
